/* 
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
#ifndef __SCHEDULER_H__
#define __SCHEDULER_H__

#include "TYPES.h"
#include "ProcessDescriptor.h"



                              /*============================*/
                              /* Author: A. GAMATIE         */
                              /* Last modified 03/13/2003   */
                              /*============================*/

/**************************************************************************************/
/*****************************   class SCHEDULER (headers) ****************************/
/**************************************************************************************/
/* The following class defines what we use as scheduler for the APEX services. It is  */
/* still under implementation. So no guarantee is given that it works correctly.      */
/* However, it should not be very far from what you expect. :-) Please, send any      */
/* remark at the following address agamatie@irisa.fr                                  */
/**************************************************************************************/

typedef class Scheduler *ptScheduler;
class Scheduler
{

  private:

   ptProcessDescriptor Range[MAX_NUMBER_OF_PROCESSES];   /* process descriptors range */
   int Current_Size;                                 /* current number of descriptors */
   LockLevel_type Lock_Level;                                  /* locklevel indicator */
   OperatingMode_type Operating_Mode;                     /* operating mode indicator */
   ptProcessDescriptor Active_Process;               /* pointer on the active process */
   ptProcessDescriptor Preempted_Process;                 /* latest preempted process */
   SystemTime_type GlobalTime;                                 /* current system time */


   
  public:

   Scheduler();
     /* Constructor */

 
   logical empty();
     /* Returns true if no process is recorded yet. Otherwise, it returns false. */


   logical full();
     /* Returns true if there is space to record a process. Otherwise, it returns
        false. */

   void clean(); 
     /* Removes all the process descriptors already recorded. */


   int getCurrentNbProc();
     /* Returns the current number of recorded processes. */

 
   ~Scheduler();
     /* Destructor */


   void ProcessSetState(ProcessID_type ProcessID, ProcessState_type state);
     /* If "ProcessID" denotes a process actually known by the scheduler, its state
        is set to the input "state" value. */


   ProcessState_type ProcessGetState(ProcessID_type ProcessID);
     /* Returns the current state of the process idenfied by "ProcessID". */


   SystemTime_type ProcessGetPeriod(ProcessID_type ProcessID);
     /* Returns the period of the process idenfied by "ProcessID". */


   void ProcessSetState_many(logical process_range[MAX_NUMBER_OF_PROCESSES], ProcessState_type state);
     /* All the processes reported in the input parameter "range" 
        (i.e, process_range[pid] is true where pid identifies a process)
        are set to the input "state" value. */


   Message_type ProcessRetrieveMessage(ProcessID_type ProcessID);
     /* Returns the message retrieved by a process (identified by ProcessID)
        which may be already blocked on a receive message request in a
        buffer. */


   void ProcessCopyMessage(Message_type message, ProcessID_type ProcessID);
     /* Copies the message denoted by "message" in the descriptor of the 
        process identified by "ProcessID". This process is previously 
        blocked on a receive message request in a buffer. */


   void ProcessSetPriority(ProcessID_type ProcessID, Priority_type Priority);
     /* Sets the priority value associated with "ProcessID" to 
        "Priority". */
 

   void ProcessSetDeadlineTime(ProcessID_type ProcessID, SystemTime_type time);
     /* Sets the deadline time value associated with "ProcessID" to 
        "time". */


   void ProcessSetAttributes(ProcessID_type ProcessID, ProcessAttributes_type attributes);
     /* Sets the attribute values associated with "ProcessID" to 
        "attributes". */


   logical ProcessRecording(ProcessName_type name);
     /* Records a new process descriptor associated with a process named
        "name". */

   ProcessID_type ProcessRecord(ProcessName_type name);
     /* This function must imperatively be removed later, it is a wrong version of the previous one, it allows to record a new process descriptor associated with a process named
        "name". */

   ProcessStatus_type ProcessGetStatus(ProcessID_type ProcessID);
     /* Returns the current status of the process idenfied by "ProcessID". */


   Priority_type ProcessGetBasicPriority(ProcessID_type ProcessID);
     /* Returns the basic priority associated with the process idenfied by
        "ProcessID". */


   SystemTime_type ProcessGetTimeCapacity(ProcessID_type ProcessID);
     /* Returns the time capacity associated with the process idenfied by
        "ProcessID". */


   logical ProcessCheckWaitIndicator(ProcessID_type ProcessID);
     /* Returns true if the process identified by "ProcessID" is
        already created. Otherwise, it returns false. */


   logical ProcessCheckName(ProcessName_type name, ProcessID_type& ProcessID);
     /* Checks whether or not a process named "name" has been already recorded.
        If it is the case, "ProcessID" denotes its identifier (it equals to 
        "i+1" where Range[i] is the pointer on the associated descriptor), and the
        function returns TRUE. Otherwise, it only returns FALSE. */


   logical ProcessCheckID(ProcessID_type ProcessID, ProcessStatus_type& status);
     /* Returns true if the process identified by "ProcessID" is
        present, and "status" is the associated status. Otherwise, it 
        returns false. */


   void ProcessResetLockLevel(logical res); 
     /* Resets the lock level. */


   void ProcessIncrementLockLevel(logical inc);
     /* Increments the lock level by one. */


   void ProcessDecrementLockLevel(logical& reschedule); 
     /* Decrement the lock level by one. */


   void ProcessReleaseResources(ProcessID_type ProcessID); 
     /* Release all the resources detained by the process identified by
       "ProcessID". NOTE: this service is not yet implemented! */


   void ProcessResetContext(ProcessID_type ProcessID);
     /* Resets the context of the process identified by "ProcessID". 
        NOTE: this service is not yet implemented! */   
 
 
   void ProcessResetErrorStatus();
     /* Resets the error status. NOTE: this service is not yet implemented! */


   OperatingMode_type ProcessCheckOperatingMode();
     /* Returns the current operating mode. */


   ProcessID_type ProcessGetActive(ProcessStatus_type& status, logical& diagnostic);
     /* Returns the active process identifier if there is an active one,
        and "status" is it associated status, finally, "ok" is true.
        Otherwise, it only set "ok" to false. */


   ProcessID_type ProcessGetHighestPriority(logical& diagnostic);
     /* Returns the process in the ready state with the highest 
        priority, and "ok" is true. Otherwise, it only set "ok"
        to false. */
 

   logical ProcessCheckPreemptionStatus(logical request);
     /* On a request (i.e, request is true), it returns true if
        preemption is enabled, otherwise it returns false. */


   logical ProcessSchedulingRequest();
     /* Tries to perform a rescheduling. If it is possible, it returns
        true, otherwise it returns false. */


   logical ProcessCheckCreated(ProcessName_type name);
     /* Returns true if the process named "name" is
        already created. Otherwise, it returns false. */
 

   void ProcessSetCreated(ProcessID_type ProcessID);
     /* Sets the creation indicator associated with the process identified
        by "ProcessID" to true. */


   logical ProcessCheckPreviousPreempted(ProcessID_type ProcessID);
     /* Returns true if the process identified by "ProcessID" has been
        previously suspended. Otherwise, it returns false. */


   void ProcessStartCounter(ProcessID_type ProcessID, SystemTime_type delay);
     /* Initializes the time counter associated with the process
        identified by "ProcessID" with the input "delay" value. */
   

   void ProcessStopCounter(ProcessID_type ProcessID);
     /* Stops the time counter associated with the process
        identified by "ProcessID". */


   void ProcessStopCounters_many(logical process_range[MAX_NUMBER_OF_PROCESSES]);
     /* Stops time counters associated with processes reported in "process_range". */  
 

   void ProcessUpdateTimeCounters(logical timedout[MAX_NUMBER_OF_PROCESSES]);
     /* Updates time counters (decremented by ONE time unit) associated
        with every process. If the counter value of a process identified
        by PID just becomes zero, timedout[PID] is set to true, otherwise
        it is false. Furthermore, the global time is incremented by one. */


   void ProcessUpdateTimeCountersProfiling(double val, logical timedout[MAX_NUMBER_OF_PROCESSES]);
     /* This is a variant of the previous method. Instead of ONE time unit
        it considers "val" time units. This value can result from a 
        profiling. So, this method updates time counters (decremented by
        "val" time unit) associated with every process. If the counter value
        of a process identified by PID just becomes zero, timedout[PID] is
        set to true, otherwise it is false. Furthermore, the global time is
        incremented by "val". */


   SystemTime_type ProcessGetTime();
     /* Returns the value of the global time. */


   logical ProcessCheckSuspensionIndicator(ProcessID_type ProcessID);
     /* Returns true if the process identified by "ProcessID" is
       already suspended (i.e, some other process suspends it using
       the "SUSPEND" service). Otherwise, it returns false. */


   void ProcessSetSuspensionIndicator(ProcessID_type ProcessID, logical suspension);
     /* Sets the suspension indicator associated with the process identified
      by "ProcessID" to the input "suspension" value. */
};

/**************************************************************************************/
/*********************************  SCHEDULER  : END  *********************************/
/**************************************************************************************/


#endif // __SCHEDULER_H__



